extern crate glfw;

use std::sync::mpsc::Receiver;

use glfw::{Context};
use glfw::Glfw;
use glfw::Window;
use glfw::WindowEvent;

//----------------------------------------------------------------------------------//
pub fn load_glfw() -> Glfw {
	let glfw_i = glfw::init(glfw::FAIL_ON_ERRORS).unwrap();
	
	glfw_i
}

//----------------------------------------------------------------------------------//
pub fn open(glfw:Glfw, width: u32, height: u32, title: String) -> WindowI {
	let (window, events) = glfw.create_window(width, height, &title, glfw::WindowMode::Windowed).unwrap();
	
	WindowI {
		instance: glfw,
		window: window,
		events: events,
	}
}


//----------------------------------------------------------------------------------//
pub fn update(mut window: WindowI) -> WindowI {
	while !window.should_close() {
		
		window = window.input();
		window = window.render();
	}
	
	window
}


//----------------------------------------------------------------------------------//
pub fn close(window: WindowI) {
	
	window.close();
}

//----------------------------------------------------------------------------------//
pub fn unload_glfw() {
	
	glfw::terminate();
}

//----------------------------------------------------------------------------------//
pub struct WindowI {
	instance: Glfw,
	window: Window,
	events: Receiver<(f64, WindowEvent)>,
}

impl WindowI {
	pub fn input(mut self) -> WindowI {
		
		self.instance.poll_events();
		self
	}
	
	pub fn render(mut self) -> WindowI {
		
		&self.window.swap_buffers();
		self
	}
	
	pub fn close(mut self){
	
		&self.window.set_should_close(true);
	}
	
	pub fn should_close(&self) -> bool {
		
		self.window.should_close()
	}
	
	pub fn events(self) -> Receiver<(f64, WindowEvent)> {
		
		self.events
	}
}

//----------------------------------------------------------------------------------//